service system/initial {
    use = system/readahead
    critical
start {

umask 022

echo "Mounting proc at /proc ..."
mkdir -p /proc && mount -n -t proc none /proc || exit $? &

echo "Mounting sysfs at /sys ..."
mkdir -p /sys && mount -n -t sysfs none /sys || exit $? &

# need /proc mounted here.
wait 

if [ -e /sbin/udevstart ]; then
    echo "Mounting ramfs at /dev ..."
    mkdir -p /dev && mount -n -t ramfs none /dev && mknod /dev/null c 1 3 -m 666 || exit $?

    echo "Configuring system to use udev ..."
    /sbin/udevstart
    
    # udevstart fails sometimes, retry until /dev/zero exists.
    while [ ! -e /dev/zero ]; do echo "waiting for /dev/zero to appere"; sleep 2; done

    #Not provided by sysfs but needed
    ln -snf /proc/self/fd /dev/fd &
    ln -snf fd/0 /dev/stdin &
    ln -snf fd/1 /dev/stdout &
    ln -snf fd/2 /dev/stderr &
    ln -snf /proc/kcore /dev/core &
    ln -snf /proc/asound/oss/sndstat /dev/sndstat &

    # Same thing as /dev/.devfsd
    touch /dev/.udev &
fi

wait
echo "Mounting devpts at /dev/pts ..."
mkdir -p /dev/pts && mount -n -t devpts -o gid=5,mode=0620 none /dev/pts || exit $? &

echo "Mounting devshm at /dev/shm ..."
mkdir -p /dev/shm && mount -n -t tmpfs none /dev/shm || exit $? &


# Setup hotplugging (if possible)
if [ -e /proc/sys/kernel/hotplug ]
then
    if   [ -x /sbin/udevsend ] ; then
	echo "  Setting /sbin/udevsend as hotplug agent ..."
	echo "/sbin/udevsend" > /proc/sys/kernel/hotplug

    elif [ -x /sbin/hotplug  ] ; then
	echo "  Using /sbin/hotplug as hotplug agent ..."
	echo "/sbin/hotplug" > /proc/sys/kernel/hotplug

    else
	echo "  Setting /sbin/udev as hotplug agent ..."
	echo "/sbin/udev" > /proc/sys/kernel/hotplug
    fi
fi

# Set the console loglevel to 1 for a cleaner boot
# the logger should anyhow dump the ring-0 buffer at start to the
# logs, and that with dmesg can be used to check for problems
/bin/dmesg -n 1 &

wait
#exit happily, or sulogin will be executed.
exit 0
}
}
