
service system/keymaps {
	need = system/initial system/mountfs

start {
	export PATH="/bin:/sbin:/usr/bin:/usr/sbin"
	if [ -e /etc/conf.d/keymaps ]; then
	    source /etc/conf.d/keymaps
	fi
	
	# Fedora stuff follows
	if [ -f /etc/sysconfig/keyboard ]; then
	    . /etc/sysconfig/keyboard
	fi
	if [ -n "$KEYTABLE" -a -d "/lib/kbd/keymaps" ]; then
	    KEYMAP="$KEYTABLE.map"
	fi
	# End of Fedora stuff

	if [ -e /etc/console/boottime.kmap.gz ]; then
	    loadkeys -q /etc/console/boottime.kmap.gz &>/dev/null
	    exit 0
	fi

	#if is_uml_sys ; then
	#	echo "Loading key mappings"
	#	return 0
	#fi

	WINDOWKEYS_KEYMAP=
    
	if [[ -z ${KEYMAP} ]] ; then
		echo "You need to setup KEYMAP in /etc/conf.d/keymaps first"
		exit 1
	fi

	# Make sure user isn't using rc.conf anymore
	if grep -qs ^KEYMAP= /etc/rc.conf ; then
		echo "KEYMAP should not be set in /etc/rc.conf but in /etc/conf.d/keymaps"
	fi


	# Force linux keycodes for PPC.
	if [[ -f /proc/sys/dev/mac_hid/keyboard_sends_linux_keycodes ]] ; then
		echo 1 > /proc/sys/dev/mac_hid/keyboard_sends_linux_keycodes
	fi

	# New kbd support.
	echo "Loading key mappings"
	if [[ -x /bin/loadkeys ]] ; then
		[[ ${SET_WINDOWKEYS} == "yes" ]] && WINDOWKEYS_KEYMAP="windowkeys"
		/bin/loadkeys -q ${WINDOWKEYS_KEYMAP} ${KEYMAP} ${EXTENDED_KEYMAPS} &>/dev/null
	else
		echo "/bin/loadkeys not found"
		exit 1
	fi

	# Set terminal encoding to either ASCII or UNICODE.
	# See utf-8(7) for more information.
	termencoding="" termmsg=""
	if [[ ${UNICODE} == "yes" ]] ; then
		dumpkey_opts=""
		[[ -n ${DUMPKEYS_CHARSET} ]] && dumpkey_opts="-c ${DUMPKEYS_CHARSET}"

		/usr/bin/kbd_mode -u
		dumpkeys ${dumpkey_opts} | loadkeys --unicode
		termencoding=$'\033%G'
		termmsg="UTF-8"
	else
		termencoding=$'\033(K'
		termmsg="ASCII"
	fi
	ttydev=""
	[[ -d /dev/vc ]] \
		&& ttydev=/dev/vc/ \
		|| ttydev=/dev/tty
	echo "Setting terminal encoding to ${termmsg} ..."
	for n in $(seq 1 11) ; do
		echo -n -e ${termencoding} > ${ttydev}${n}
	done
}
}

# vim:ts=4
