service system/mountroot {
	need = system/initial
	critical
start {

	export PATH="/bin:/usr/bin:/sbin:/usr/sbin"
	retval=0

	# startDM
	if [ -f /sbin/dmsetup ]; then
	    /sbin/dmsetup mknodes >/dev/null 2>&1 
	fi
	
	#startLVM
	if [ -f /sbin/vgchange ]; then
	    # try to load module in case that hasn't been done yet
	    modprobe lvm >/dev/null 2>&1
	    modprobe lvm-mod >/dev/null 2>&1
	    modprobe dm-mod >/dev/null 2>&1
	    mknod --mode=600 /dev/lvm c 109 0 >/dev/null 2>&1 || true
	    dm_dir="/dev/mapper"
	    dm_file="$dm_dir/control"
	    if [ ! -f /dev/.devfsd ]; then
		major=$(grep "[0-9] misc$" /proc/devices | sed 's/[ ]\+misc//')
		minor=$(grep "[0-9] device-mapper$" /proc/misc | sed 's/[ ]\+device-mapper//')
		test -d $dm_dir || mkdir --mode=755 $dm_dir >/dev/null 2>&1
		test -c $dm_file -o -z "$major" -o -z "$minor" || mknod --mode=600 $dm_file c $major $minor >/dev/null 2>&1
	    fi
	    /sbin/vgscan --ignorelockingfailure --mknodes >/dev/null 2>&1
	    /sbin/vgchange --ignorelockingfailure -a y >/dev/null 2>&1
	fi
	
	# startEVMS
	if [ -f /sbin/evms_activate ]; then
	    evms_activate >/dev/null 2>&1 
	fi
		
	# checkROOT
	if [ ! -f /fastboot ] ; then
	    echo "Remounting root filesystem read-only (if necessary) ..."
	    mount -n -o remount,ro / >/dev/null 2>&1 

	    if [ -f /forcefsck ] ; then
		echo "Checking root filesystem (full fsck forced)"
		/sbin/fsck -C -a -f /
		# /forcefsck isn't deleted because system/mountfs need it.
		# it'll be deleted in that script.
		retval=$?
	    else

		# Obey the fs_passno setting for / (see fstab(5))
		# - find the / entry
		# - make sure we have 6 fields
		# - see if fs_passno is something other than 0
		ROOTLINE=$(cat /etc/fstab | while read fs mountpoint typ options dump pass junk ; do if [[    "${fs}" == /* && "${mountpoint}" == "/" && "${pass}" != "0" ]] ; then echo ${fs} ${mountpoint} ${typ} ${options} ${dump} ${pass} ; fi done) # */
		if [ -n "$ROOTLINE" ]; then
		    echo "Checking root filesystem ..."
		    /sbin/fsck -C -T -a /
		    retval=$?
		else
		    echo "Skipping root filesystem check (fstab's passno == 0) ..."
		    retval=0
		fi
		
	    fi
	    
	    if [ "${retval}" -eq 0 ] ; then
		echo "Filsystem check on / sucseeded, now mounting ..."
	    elif [ "${retval}" -eq 1 ] ; then
		echo "Filesystem repaired, now mounting ..."
	    elif [ "${retval}" -eq 2 ] || [ "${retval}" -eq 3 ] ; then
		echo "Filesystem repaired, but reboot needed!"
		echo -ne "\a"; sleep 1; echo -ne "\a"; sleep 1
		echo -ne "\a"; sleep 1; echo -ne "\a"; sleep 1
		echo "Rebooting in 2 seconds ..."
		sleep 2
		echo "Rebooting"
		/sbin/ngc -j reboot
	    else
		echo "Filesystem couldn't be fixed :("
		#exit 1, with critical set, will shut down initng, and execute sulogin.
		exit 1
	    fi
	fi
	echo "Shud mount / now."

	# Should we mount root rw ?
	if mount -vf -o remount / | grep "\(.*rw.*\)" 2>&1 >/dev/null
	then

	    echo "Remounting root filesystem read/write"
	    mount -n -o remount,rw / >/dev/null 2>&1 
	    if [ "$?" -ne 0 ]
	    then
		echo "Root filesystem could not be mounted read/write :("
		exit 1
	    fi
	fi

	#
	# Create /etc/mtab
	#
	# Don't create mtab if /etc is readonly
	if ! touch /etc/mtab 2>/dev/null
	then
		echo "Skipping /etc/mtab initialization (ro root?)"
		exit 0
	fi

	# Clear the existing mtab
	>/etc/mtab

	# Add the entry for / to mtab
	mount -f /

	# Don't list root more than once
	grep -v " / " /proc/mounts >>/etc/mtab
	# Now make sure /etc/mtab have additional info (gid, etc) in there

	cat /proc/mounts | while read pm_fs pm_mountpoint pm_junk ; do
	  cat /etc/fstab | while read ef_fs ef_mountpoint ef_junk ; do
	    if [ "${pm_mountpoint}" = "${ef_mountpoint}" ] ; then
	      echo "Mounting ${pm_mountpoint} rw ..."
              mount -f -o remount "${pm_mountpoint}"
              continue
            fi
          done
        done

	# Remove stale backups
	rm -f /etc/mtab~ /etc/mtab~~ &> /dev/null
	# Return Happily., or sulogin will be executed.
	exit 0
}
}
